@@ -34,15 +34,25 @@ using namespace std;
 #include "msg/SimpleMessenger.h"
 
 #include "common/Timer.h"
-#include "global/global_init.h"
 #include "common/ceph_argparse.h"
 
+#include "global/global_init.h"
+#include "global/signal_handler.h"
+
 #include "include/color.h"
 #include "common/errno.h"
 #include "common/pick_address.h"
 
 #include "perfglue/heap_profiler.h"
 
+OSD *osd = NULL;
+
+void handle_osd_signal(int signum)
+{
+  if (osd)
+    osd->handle_signal(signum);
+}
+
 void usage() 
 {
   derr << "usage: ceph-osd -i osdid [--osd-data=path] [--osd-journal=path] "
@@ -57,7 +67,7 @@ int main(int argc, const char **argv)
   argv_to_vec(argc, argv, args);
   env_to_vec(args);
 
-  global_init(args, CEPH_ENTITY_TYPE_OSD, CODE_ENVIRONMENT_DAEMON, 0);
+  global_init(NULL, args, CEPH_ENTITY_TYPE_OSD, CODE_ENVIRONMENT_DAEMON, 0);
   ceph_heap_profiler_init();
 
   // osd specific args
@@ -290,43 +300,31 @@ int main(int argc, const char **argv)
 	 << TEXT_NORMAL << dendl;
   }
 
-  SimpleMessenger *client_messenger = new SimpleMessenger(g_ceph_context);
-  SimpleMessenger *cluster_messenger = new SimpleMessenger(g_ceph_context);
-  SimpleMessenger *messenger_hbin = new SimpleMessenger(g_ceph_context);
-  SimpleMessenger *messenger_hbout = new SimpleMessenger(g_ceph_context);
+  SimpleMessenger *client_messenger = new SimpleMessenger(g_ceph_context,
+                                                          entity_name_t::OSD(whoami),
+                                                          getpid());
+  SimpleMessenger *cluster_messenger = new SimpleMessenger(g_ceph_context,
+                                                           entity_name_t::OSD(whoami),
+                                                           getpid());
+  SimpleMessenger *messenger_hbin = new SimpleMessenger(g_ceph_context,
+                                                        entity_name_t::OSD(whoami),
+                                                        getpid());
+  SimpleMessenger *messenger_hbout = new SimpleMessenger(g_ceph_context,
+                                                         entity_name_t::OSD(whoami),
+                                                         getpid());
   cluster_messenger->set_cluster_protocol(CEPH_OSD_PROTOCOL);
   messenger_hbin->set_cluster_protocol(CEPH_OSD_PROTOCOL);
   messenger_hbout->set_cluster_protocol(CEPH_OSD_PROTOCOL);
 
-  r = client_messenger->bind(g_conf->public_addr, getpid());
-  if (r < 0)
-    exit(1);
-  r = cluster_messenger->bind(g_conf->cluster_addr, getpid());
-  if (r < 0)
-    exit(1);
-
-  // hb should bind to same ip as cluster_addr (if specified)
-  entity_addr_t hb_addr = g_conf->cluster_addr;
-  if (!hb_addr.is_blank_ip())
-    hb_addr.set_port(0);
-  r = messenger_hbout->bind(hb_addr, getpid());
-  if (r < 0)
-    exit(1);
-
   global_print_banner();
 
   cout << "starting osd." << whoami
-       << " at " << client_messenger->get_ms_addr()
+       << " at " << client_messenger->get_myaddr()
        << " osd_data " << g_conf->osd_data
        << " " << ((g_conf->osd_journal.empty()) ?
 		    "(no journal)" : g_conf->osd_journal)
        << std::endl;
 
-  client_messenger->register_entity(entity_name_t::OSD(whoami));
-  cluster_messenger->register_entity(entity_name_t::OSD(whoami));
-  messenger_hbin->register_entity(entity_name_t::OSD(whoami));
-  messenger_hbout->register_entity(entity_name_t::OSD(whoami));
-
   Throttle client_throttler(g_conf->osd_client_message_size_cap);
 
   uint64_t supported =
@@ -334,28 +332,43 @@ int main(int argc, const char **argv)
     CEPH_FEATURE_NOSRCADDR |
     CEPH_FEATURE_PGID64;
 
-  client_messenger->set_default_policy(SimpleMessenger::Policy::stateless_server(supported, 0));
+  client_messenger->set_default_policy(Messenger::Policy::stateless_server(supported, 0));
   client_messenger->set_policy(entity_name_t::TYPE_CLIENT,
-			       SimpleMessenger::Policy::stateless_server(supported, 0));
+			       Messenger::Policy::stateless_server(supported, 0));
   client_messenger->set_policy_throttler(entity_name_t::TYPE_CLIENT, &client_throttler);
   client_messenger->set_policy(entity_name_t::TYPE_MON,
-                               SimpleMessenger::Policy::client(supported,
-                                                               CEPH_FEATURE_UID |
-							       CEPH_FEATURE_PGID64 |
-							       CEPH_FEATURE_OSDENC));
+                               Messenger::Policy::client(supported,
+							 CEPH_FEATURE_UID |
+							 CEPH_FEATURE_PGID64 |
+							 CEPH_FEATURE_OSDENC));
   //try to poison pill any OSD connections on the wrong address
   client_messenger->set_policy(entity_name_t::TYPE_OSD,
-			       SimpleMessenger::Policy::stateless_server(0,0));
-
-  cluster_messenger->set_default_policy(SimpleMessenger::Policy::stateless_server(0, 0));
-  cluster_messenger->set_policy(entity_name_t::TYPE_MON, SimpleMessenger::Policy::client(0,0));
+			       Messenger::Policy::stateless_server(0,0));
+  
+  cluster_messenger->set_default_policy(Messenger::Policy::stateless_server(0, 0));
+  cluster_messenger->set_policy(entity_name_t::TYPE_MON, Messenger::Policy::client(0,0));
   cluster_messenger->set_policy(entity_name_t::TYPE_OSD,
-				SimpleMessenger::Policy::lossless_peer(supported,
-								       CEPH_FEATURE_UID |
-								       CEPH_FEATURE_PGID64 |
-								       CEPH_FEATURE_OSDENC));
+				Messenger::Policy::lossless_peer(supported,
+								 CEPH_FEATURE_UID |
+								 CEPH_FEATURE_PGID64 |
+								 CEPH_FEATURE_OSDENC));
   cluster_messenger->set_policy(entity_name_t::TYPE_CLIENT,
-				SimpleMessenger::Policy::stateless_server(0, 0));
+				Messenger::Policy::stateless_server(0, 0));
+
+  r = client_messenger->bind(g_conf->public_addr);
+  if (r < 0)
+    exit(1);
+  r = cluster_messenger->bind(g_conf->cluster_addr);
+  if (r < 0)
+    exit(1);
+
+  // hb should bind to same ip as cluster_addr (if specified)
+  entity_addr_t hb_addr = g_conf->cluster_addr;
+  if (!hb_addr.is_blank_ip())
+    hb_addr.set_port(0);
+  r = messenger_hbout->bind(hb_addr);
+  if (r < 0)
+    exit(1);
 
   // Set up crypto, daemonize, etc.
   // Leave stderr open in case we need to report errors.
@@ -374,13 +387,13 @@ int main(int argc, const char **argv)
     return -1;
   global_init_chdir(g_ceph_context);
 
-  OSD *osd = new OSD(whoami, cluster_messenger, client_messenger,
-		     messenger_hbin, messenger_hbout,
-		     &mc,
-		     g_conf->osd_data, g_conf->osd_journal);
+  osd = new OSD(whoami, cluster_messenger, client_messenger,
+		messenger_hbin, messenger_hbout,
+		&mc,
+		g_conf->osd_data, g_conf->osd_journal);
   err = osd->pre_init();
   if (err < 0) {
-    derr << TEXT_RED << " ** ERROR: initializing osd failed: " << cpp_strerror(-err)
+    derr << TEXT_RED << " ** ERROR: osd pre_init failed: " << cpp_strerror(-err)
 	 << TEXT_NORMAL << dendl;
     return 1;
   }
@@ -389,14 +402,20 @@ int main(int argc, const char **argv)
   global_init_shutdown_stderr(g_ceph_context);
 
   client_messenger->start();
-  messenger_hbin->start_with_nonce(getpid());
+  messenger_hbin->start();
   messenger_hbout->start();
   cluster_messenger->start();
 
+  // install signal handlers
+  init_async_signal_handler();
+  register_async_signal_handler(SIGHUP, sighup_handler);
+  register_async_signal_handler_oneshot(SIGINT, handle_osd_signal);
+  register_async_signal_handler_oneshot(SIGTERM, handle_osd_signal);
+
   // start osd
   err = osd->init();
   if (err < 0) {
-    derr << TEXT_RED << " ** ERROR: initializing osd failed: " << cpp_strerror(-err)
+    derr << TEXT_RED << " ** ERROR: osd init failed: " << cpp_strerror(-err)
          << TEXT_NORMAL << dendl;
     return 1;
   }
@@ -406,12 +425,16 @@ int main(int argc, const char **argv)
   messenger_hbout->wait();
   cluster_messenger->wait();
 
+  unregister_async_signal_handler(SIGHUP, sighup_handler);
+  unregister_async_signal_handler(SIGINT, handle_osd_signal);
+  unregister_async_signal_handler(SIGTERM, handle_osd_signal);
+
   // done
   delete osd;
-  client_messenger->destroy();
-  messenger_hbin->destroy();
-  messenger_hbout->destroy();
-  cluster_messenger->destroy();
+  delete client_messenger;
+  delete messenger_hbin;
+  delete messenger_hbout;
+  delete cluster_messenger;
 
   // cd on exit, so that gmon.out (if any) goes into a separate directory for each node.
   char s[20];
