@@ -16,6 +16,7 @@ using namespace std;
 #include "rgw_user.h"
 #include "rgw_access.h"
 #include "rgw_acl.h"
+#include "rgw_acl_s3.h"
 #include "rgw_log.h"
 #include "rgw_formats.h"
 #include "auth/Crypto.h"
@@ -430,6 +431,33 @@ static void remove_old_indexes(RGWUserInfo& old_info, RGWUserInfo new_info)
     cerr << "ERROR: this should be fixed manually!" << std::endl;
 }
 
+static bool char_is_unreserved_url(char c)
+{
+  if (isalnum(c))
+    return true;
+
+  switch (c) {
+  case '-':
+  case '.':
+  case '_':
+  case '~':
+    return true;
+  default:
+    return false;
+  }
+}
+
+static bool validate_access_key(string& key)
+{
+  const char *p = key.c_str();
+  while (*p) {
+    if (!char_is_unreserved_url(*p))
+      return false;
+    p++;
+  }
+  return true;
+}
+
 int bucket_stats(rgw_bucket& bucket, Formatter *formatter)
 {
   RGWBucketInfo bucket_info;
@@ -478,7 +506,7 @@ int main(int argc, char **argv)
   argv_to_vec(argc, (const char **)argv, args);
   env_to_vec(args);
 
-  global_init(args, CEPH_ENTITY_TYPE_CLIENT, CODE_ENVIRONMENT_UTILITY, 0);
+  global_init(NULL, args, CEPH_ENTITY_TYPE_CLIENT, CODE_ENVIRONMENT_UTILITY, 0);
   common_init_finish(g_ceph_context);
 
   std::string user_id, access_key, secret_key, user_email, display_name;
@@ -810,6 +838,10 @@ int main(int argc, char **argv)
       access_key.append(subuser);
     }
     if ((!access_key.empty()) && (!secret_key.empty())) {
+      if (key_type == KEY_TYPE_S3 && !validate_access_key(access_key)) {
+        cerr << "access key contains illegal characters" << std::endl;
+        return 1;
+      }
       RGWAccessKey k;
       k.id = access_key;
       k.key = secret_key;
@@ -910,7 +942,7 @@ int main(int argc, char **argv)
     rgw_obj obj(bucket, object);
     int ret = store->get_attr(NULL, obj, RGW_ATTR_ACL, bl);
 
-    RGWAccessControlPolicy policy;
+    RGWAccessControlPolicy_S3 policy;
     if (ret >= 0) {
       bufferlist::iterator iter = bl.begin();
       try {
